@@ -7,13 +7,14 @@ use Encode;
 use DBI qw(:utils);
 
 use Zonemaster::Backend::Config;
+use Zonemaster::Backend::DB::PostgreSQL;
 
-die "The configuration file does not contain the PostgreSQL backend" unless (lc(Zonemaster::Backend::Config->load_config()->BackendDBType()) eq 'postgresql');
-my $db_user = Zonemaster::Backend::Config->load_config()->DB_user();
-my $db_password = Zonemaster::Backend::Config->load_config()->DB_password();
-my $connection_string = Zonemaster::Backend::Config->load_config()->DB_connection_string();
-
-my $dbh = DBI->connect( $connection_string, $db_user, $db_password, { RaiseError => 1, AutoCommit => 1 } );
+my $config = Zonemaster::Backend::Config->load_config();
+if ( $config->DB_engine ne 'PostgreSQL' ) {
+    die "The configuration file does not contain the MySQL backend";
+}
+my $dbh     = Zonemaster::Backend::DB::PostgreSQL->new( { config => $config } )->dbh;
+my $db_user = $config->POSTGRESQL_user;
 
 sub create_db {
 
